/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is ForumZilla 0.1.1.
 *
 * The Initial Developer of the Original Code is Zapogee, Inc.
 * Portions created by Zapogee, Inc. are Copyright (C) 2001
 * Zapogee, Inc.  All Rights Reserved.
 *
 * Contributor(s):
 *   Myk Melez, <myk@melez.com>, original author
 */

// Load and cache the subscriptions and items data sources so they're available
// when we need them.
getSubscriptionsDS();
getItemsDS();

var feedService = new FeedService();

// Wait a few seconds before starting Forumzilla so we don't grab the UI thread
// before the host application has a chance to display itself.  Also, starting
// on load used to crash the host app, although that's not a problem anymore.
//addEventListener("load", onLoad, false);
window.setTimeout(onLoad, 2000);

function onLoad() {
    // Make sure the storage folder exists.
    try {
        msgFolder = GetMsgFolderFromUri(feedService.storageFolder, true);
        if (!msgFolder)
            createDefaultFolder();
    }
    catch(e) {
        debug("error making sure the default folder exists: " + e);
        debug("this could be bug 248564: http://bugzilla.mozilla.org/show_bug.cgi?id=248564");
    }
  
    // Make sure the subscriptions and items data source is loaded, since we'll
    // need them for everything we do.  If it's not loaded, recheck it every
    // second until it's been thirty seconds since we started up, then give up.
    // XXX We should just set a listener to alert us when the load is done.
    var itemsDS =
        getItemsDS()
            .QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    if (!itemsDS.loaded) {
        if (new Date() - gFzStartupTime < 30 * 1000) {
            window.setTimeout(onLoad, 1000);
            return;
        }
        else
            throw("couldn't load the items datasource within thirty seconds");
    }
    var subsDS =
        getSubscriptionsDS()
            .QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    if (!subsDS.loaded) {
        if (new Date() - gFzStartupTime < 30 * 1000) {
            window.setTimeout(onLoad, 1000);
            return;
        }
        else
            throw("couldn't load the subscriptions datasource within thirty seconds");
    }
  
    // Migrate old subscriptions into the new subscriptions file.
    var oldSubs = getOldSubscriptionsFile();
    if (oldSubs.exists())
        migrateSubscriptions(oldSubs);
  
    feedService.initFeedPolling();
}

function createDefaultFolder() {
    debug("storage folder " + feedService.storageFolder + " doesn't exist; attempting to create it");
    var path = feedService.storageFolder.match(/^(.+)\/([^\/]+)$/);
    var parent = path[1];
    var child = path[2];
    if (parent && child) {
        debug("parent folder: " + parent + ", child folder: " + child);
        var parentFolder = GetMsgFolderFromUri(parent, true);
        if (parentFolder)
            parentFolder.createSubfolder(child, msgWindow);
        else
            debug("parent folder doesn't exist either!");
    }
    else {
        debug("couldn't extract parent folder URI and child name from default folder URI");
    }
    debug("created default folder");
}

function migrateSubscriptions(oldFile) {
  var oldFile2 = new LocalFile(oldFile, MODE_RDONLY | MODE_CREATE);
  var subscriptions = oldFile2.read();

  if (subscriptions && subscriptions.length > 0) {
    subscriptions = subscriptions.split(/[\r\n]+/);
    var ds = getSubscriptionsDS();
    var feeds = getSubscriptionsList();
    var url, quickMode;

    for ( var i=0 ; i<subscriptions.length ; i++ ) {
      url = subscriptions[i];
      quickMode = "false";

      // Trim whitespace around the URL.
      url = url.replace(/^\s+/, "");
      url = url.replace(/\s+$/, "");

      // If the URL is prefixed by a dollar sign, the feed is in quick mode,
      // which means it won't download item content but rather construct a message
      // from whatever information is available in the feed file itself
      // (i.e. content, description, etc.).
      if (url[0] == '$') {
        quickMode = "true";
        url = url.substr(1);
        url = url.replace(/^\s+/, "");
      }

      // Ignore blank lines and comments.
      if (url.length == 0 || url[0] == "#")
        continue;

      if (feeds.IndexOf(rdf.GetResource(url)) == -1) {
        // Add the feed to the list.
        id = rdf.GetResource(url);
        feeds.AppendElement(id);
        ds.Assert(id, RDF_TYPE, FZ_FEED, true);
        ds.Assert(id, DC_IDENTIFIER, rdf.GetLiteral(url), true);
        ds.Assert(id, FZ_QUICKMODE, rdf.GetLiteral(quickMode), true);
      }
    }
  }
  oldFile2.close();
  oldFile.moveTo(null, "subscriptions.txt.bak");
}

function getOldSubscriptionsFile() {
  // Get the app directory service so we can look up the user's profile dir.
  var appDirectoryService =
    Components
      .classes["@mozilla.org/file/directory_service;1"]
        .getService(Components.interfaces.nsIProperties);
  if ( !appDirectoryService )
    throw("couldn't retrieve the directory service");

  // Get the user's profile directory.
  var profileDir =
    appDirectoryService.get("ProfD", Components.interfaces.nsIFile);
  if ( !profileDir )
    throw ("couldn't retrieve the user's profile directory");

  // Get the user's subscriptions file.
  var subscriptionsFile = profileDir.clone();
  subscriptionsFile.append("subscriptions.txt");

  return subscriptionsFile;
}

function getMessageWindow() {
  return msgWindow;
}
